#!/usr/bin/perl
#
# SCoRE - Search a Corpus for Regular Expressions
#
# Perl module for general routines
#
# (c) Matthew Purver, King's College London, 2001
#
# $Log: SCoRE.pm,v $
# Revision 1.1  2001/11/09 12:30:59  purver
# Initial revision
#

package SCoRE;

# export subroutines
require Exporter;
our @ISA = qw( Exporter );
our @EXPORT = qw( processCLArgs processCGIVars 
		  getFileList writeListFile closeListFile 
		  startDisplay stopDisplay );
our @EXPORT_OK = qw( $corpusPath $corpusFilePath $corpuListPath $scorePath $webPath
		     $saveDir $saveFile $tempFile $fileExt 
		     $caseSens $punctSens $useTurns $nStart $nEnd @fileTypes $startDir
		     $nBack $nFor $showAll $sentNum $printHTML );


# local settings
$corpusPath = 'BNC_PATH';			# path to corpus
$corpusFilePath = "$corpusPath/Texts";			# path to corpus files
$corpusListPath = 'LIST_PATH';				# path to corpus file lists
$webPath = 'WEB_PATH';					# path to CGI scripts as seen by web server
$scorePath = 'SCORE_PATH';	# path to Perl scripts
$tempFile = 'tmp.html';					# temporary results file
$saveDir = 'searches';					# dir for saving search results
$saveFile = 'saved.html';				# file listing saved searches
$fileExt = '.html';					# file extension for saved searches

# pattern matching settings
$caseSens = 0;		# case sensitivity
$punctSens = 0;		# punctuation sensitivity
$useTurns = 0;		# 0 = sentences, 1 = turns
$nStart = 0;		# sent/turn number to start at
$nEnd = 1e06;		# sent/turn number to end at
@fileTypes = ();	# include/exclude file types
$fileType = '+spolog2';	# default include/exclude type

# display settings
$nBack = 2;		# backward context number of sents/turns
$nFor = 2;		# forward context number of sents/turns
$showAll = 0;		# 0 = matches only, 1 = whole file
$sentNum = 0;		# 1 for sentence numbering
$printHTML = 1;		# HTML display


# get switches from command-line args
sub processCLArgs {

    my ( @args ) = @_;
    my $matchstring = undef;

    # set variables as required
    while ( my $arg = shift( @args ) )
    { 
	( $arg =~ /^-a/ ) and ( $showAll = 1 );
	( $arg =~ /^-c/ ) and ( $caseSens = 1 );
	( $arg =~ /^-h/ ) and ( $printHTML = 1 );
	( $arg =~ /^-n/ ) and ( $sentNum = 1 );
	( $arg =~ /^-p/ ) and ( $punctSens = 1 );
	( $arg =~ /^-t/ ) and ( $useTurns = 1 );
	( $arg =~ s/^-i// ) and push( @fileTypes, "+$arg" );
	( $arg =~ s/^-x// ) and push( @fileTypes, "-$arg" );
	( $arg =~ s/^-d// ) and ( ( $arg eq '' ) or ( $startDir = $arg ) );
	( $arg =~ s/^-s// ) and ( ( $arg eq '' ) or ( $nStart = $arg ) );
	( $arg =~ s/^-e// ) and ( ( $arg eq '' ) or ( $nEnd = $arg ) );
	( $arg =~ s/^-b// ) and ( ( $arg eq '' ) or ( $nBack = $arg - 1 ) );
	( $arg =~ s/^-f// ) and ( ( $arg eq '' ) or ( $nFor = $arg ) );
	( $arg =~ s/^-m// ) and ( $matchstring = $arg );
    }

    # set default type if necessary
    defined( @fileTypes ) or ( @fileTypes = ( $fileType ) );

    # tidy up directory spec
    $startDir = "*/*/*" if ( ( not defined $startDir ) or ( $startDir eq '' ) );
    if ( $startDir =~ /(\w)(\/\w\w)?(\/\w\w\w)?/ ) {
	$startDir = $1 . ( defined( $2 ) ? $2 : "/*" ) . ( defined( $3 ) ? $3 : "/*" );
    }

    # return the match string
    return $matchstring;

}


# get relevant file names from list file or prepare to write list file
sub getFileList {
    my $writeList = 0;
    my @files = ();

    # if list file exists, get all relevant filenames
    ( my $listFile = "$corpusListPath/@fileTypes" ) =~ s/\s//g;
    if ( -f $listFile ) {
	open LISTFILE, "< $listFile" or die "Can't open $listFile";
	while ( my $file = <LISTFILE> ) {
	    chomp( $file );
	    push( @files, $file ) if ( $file =~ /$corpusFilePath\/$startDir/ );
	}
	close LISTFILE;
    }
    # otherwise find files and prepare to write list file
    else {
	@files = <$corpusFilePath/$startDir>;
	if ( $startDir eq "*/*/*" ) {
	    $writeList = 1;
	    open LISTFILE, "> $listFile" or die "Can't open $listFile";
	}
    }

    return ( $writeList, @files );
}


# write a line to list file
sub writeListFile {
    my ( $file ) = @_;
    print LISTFILE "$file\n";
}


# close list file
sub closeListFile {
    close LISTFILE;
}


# process parameters from HTML form into suitable command-line switches
sub processCGIVars {

    use CGI qw( param cgi_error header start_html end_html strong );

    # start HTML display
    print header();

    # trap errors
    my $error = cgi_error();
    if ( $error ) {
	print start_html( 'Error' ), strong( $error ), end_html();
	return;
    }

    # get user parameters
    my ( $params, $file ) = ( '', undef );
    my @names = param();
    while ( my $name = shift( @names ) ) {
	my @param = param( $name );
	for ( @param ) {
	    ( $name =~ /backward/ ) and ( s/^/-b/ );
	    ( $name =~ /forward/ ) and ( s/^/-f/ );
	    ( $name =~ /include/ ) and ( s/^/-i/ );
	    ( $name =~ /exclude/ ) and ( s/^/-x/ );
	    ( $name =~ /dir/ ) and ( s/^/-d/ );
	    ( $name =~ /start/ ) and ( s/^/-s/ );
	    ( $name =~ /end/ ) and ( s/^/-e/ );
	    ( $name =~ /string/ ) and ( s/^(.*)$/-m\"$1\"/ );
	    ( $name =~ /bnc/ ) and do {
		$file = $name;
		print "\n<H1>$file</H1>\n";
		( s/^((Whole)|(View)).*$/-a/ ) or ( s/^.*$// );
	    };
	}
	$params .= " @param";
    }
    # append file if present
    defined( $file ) and ( $params .= " < $file" );

    return $params;

}


sub startDisplay {
    my ( $title, $stream ) = @_;
    defined( $stream ) or ( $stream = STDOUT );

    print $stream "<HTML>\n<HEAD>\n<TITLE>$title</TITLE>\n</HEAD>\n\n";
    print $stream "<BODY bgcolor='Silver' text='Black'>\n";
}


sub stopDisplay {
    my ( $stream ) = @_;
    defined( $stream ) or ( $stream = STDOUT );

    print $stream <<END;
<HR>
<table width=100%>
<tr>
<td width=50% align=left><i>This page, search engine &copy; Matthew Purver, 2001</i></td>
<td width=50% align=right><i>Problems, comments, suggestions: <a href=mailto:matthew.purver\@kcl.ac.uk>matthew.purver\@kcl.ac.uk</a></i></td>
</tr>
</table>

</BODY>
</HTML>
END
}


# ensure package returns 1 to keep Perl happy
1;
